/*
* This file is part of wxSmith plugin for Code::Blocks Studio
* Copyright (C) 2006-2007  Bartlomiej Swiecki
*
* wxSmith is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 3 of the License, or
* (at your option) any later version.
*
* wxSmith is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with wxSmith. If not, see <http://www.gnu.org/licenses/>.
*
* $Revision: 7835 $
* $Id: wxsexproperties.h 7835 2012-02-22 14:40:26Z mortenmacfly $
* $HeadURL: https://svn.code.sf.net/p/codeblocks/code/branches/release-25.03/src/plugins/contrib/wxSmith/wxwidgets/wxsexproperties.h $
*/

#ifndef WXSEXPROPERTIES_H
#define WXSEXPROPERTIES_H

#include "properties/wxsarraystringcheckproperty.h"
#include "properties/wxsbitmapiconproperty.h"
#include "properties/wxscolourproperty.h"
#include "properties/wxsdimensionproperty.h"
#include "properties/wxsfontproperty.h"
#include "properties/wxspositionsizeproperty.h"
#include "properties/wxssizerflagsproperty.h"
#include "properties/wxsstyleproperty.h"
#include "properties/wxsimageproperty.h"
#include "properties/wxsimagelistproperty.h"
#include "properties/wxsimagecomboproperty.h"
#include "properties/wxsimagetreeproperty.h"

#endif
